    /*******************************************************************/
    /*
      COPYRIGHT: 1992 by SP&AR&C HOLLAND
      FILE: ssort.cpp
      DESCRIPTION : main of sorting program
      PROJECT : TFORTH
      AUTOR : A vd Horst
      LANGUAGE : Turbo C++ (2.0)
      CREATED : 920904 AH
      LAST CHANGE : added separators, and argument handling (not yet used)
                                                                       */
    /*******************************************************************/

    /* SSORT [-L] [-{S|R|E} SEP] [SOURCE-FILE [DESTINATION-FILE]]
     * sorts lines from SOURCE-FILE to DESTINATION-FILE
     * If latter exists, it is overwritten; if it does not
     * exist, SSORT will create it if possible.
     * If DESTINATION-FILE or both are missing SSORT acts as
     * a filter.
     * Options :
     * the following options specify that records to be sorted are not
     * lines, by defining a separator
     * option letters are case insensitive
     *  -C SEP is a character specificied in hex that is used to separate
     *          records
     *  -S SEP is a string that indicates the start of a new record
     *  -E SEP is a regular expression that indicates the start of a new
     *    record. A leading '^' (start of line) is considered context is
     *    and is not part of the next record
     *  -L use lexicographical ordering: ignore everything but letters,
     *    ignore case.
     *  If the -S separator is missing after the last record (or the last
     *  '\n' if no separator options specified) it is added by the program.
     *
     */



    #include <process.h>    // for exit()
    #include <fstream.h>    // for ifstream, ofstream
    #include <stdio.h>
    #include <ctype.h>
    #include <assert.h>
    #include <string.h>

    #include "loctypes.h"
    #include "lib.h"
    #include "search.h"
    #include "scompare.h"
    #include "sheap.h"
    #include "sstring.h"
    #include "sbuffer.h"

    /*******************************************************************/

    #define MAX_LINE (256)

    /*******************************************************************/

    /* The file is considered binary */
    BOOL binaryfile = FALSE;

    void main(int argc, char* argv[])  // access command-line arguments
    {
       STRING arg;         /* The argument currently under investigation */

       ifstream source;    // declare input and output streams
       ofstream dest;
       int nfiles=0;          // Number of file arguments

       // Default configuration, superfluous, for reference only
    // binaryfile = FALSE;
    // initseparatorC( "\n" );
    // initstrcmp();
    // initstartingcol("0");

       // Interpret arguments
       while ( arg =*++argv ) // Pop next argument
       {
          if ( arg[0] == '-' || arg[0] == '/' )
          {
              char option = toupper(arg[1]);

              // Most options need an argument :
              if ( option!='L' && option!='R' && arg[2]!='\0' )
                 arg = &arg[2];
              else
              {
                 arg = *++argv;
              }

              switch( option )
              {
                 case 'L' :
                   initstrcmpL();
                   break;

                 case '+' :
                   if ( !arg ) error("Specify argument for option ");
                   error("Cannot handle column start");
                   break;

                 case 'R' :
                   initstrcmpR();
                   break;

                 case 'C' :
                    if ( !arg ) error("Specify argument for option ");
                     initseparatorC( arg );
                     binaryfile = TRUE;
                     break;

                 case 'S' :
                     if ( !arg ) error("Specify argument for option ");
                     initseparatorS( arg );
                     binaryfile = FALSE;
                     break;

                 case 'E' :
                     if ( !arg ) error("Specify argument for option ");
                     initseparatorE( arg );
                     binaryfile = FALSE;
                     break;

                 case 'F' :
                     if ( !arg ) error("Specify argument for option ");
                     initseparatorF( arg );
                     binaryfile = TRUE;
                     break;

                 default  : error("Wrong option");
              }
          }
          else if ( nfiles==0 )
          {
             source.open( arg, ios::nocreate | (binaryfile?ios::binary:0) );
             if ( !source ) error("Wrong input file spec's");
             nfiles += 1;
          }
          else if ( nfiles==1)
          {
            // dest file may be overwritten
             dest.open(arg, binaryfile?ios::binary:0 );
             if ( !dest ) error("Wrong output file spec's");
             nfiles += 1;
          }
          else
              error("Too many file spec's");
       }

       if (nfiles != 2)      // test number of arguments
          error("I want 2 files");

    // The following does not work, but would be preferable
    // if ( !source ) source = stdin;
    // if ( !dest ) dest = stdout;

       SortedHeap< pSortBuffer > sortedheap;

       pSortBuffer psb;
       BOOL first = TRUE;
       do {
          psb.sb = new SortBuffer(&source, &first);
          sortedheap.additem(psb);
          cout<<'.';
       } while ( !psb.sb->eof );
       cout<<"\nwe hebben alles\n";

       while( !sortedheap.empty() )
       {
          psb = sortedheap.removeitem();
          SortString ss = psb.sb->removeitem();
    //DEBUG  cout.write( ss.start, ss.len );
          dest.write( ss.start, ss.len );
          if ( !psb.sb->empty() )
          {
             sortedheap.additem(psb);
          }
          else
             cout<<'.';
       }
       cout << "SSORT completed\n";

       source.close();        // close both streams
       dest.close();
       exit(0);
    }
